OBJ
  mb: "mbox16"

CON
  STATE_LEN = 16

VAR
  long par_exp_key[240] '''[KEY_SIZE * (NUM_ROUNDS + 1)]

'
' -------------------------------------------------------
'
VAR
  byte cog 'Cog ID

PUB start(p_mbox_in_addr, p_mbox_out_addr, p_key_size, p_key_addr) : success
{{ Start AES encoder in new cog; return True if successful.
   Parameters:
     p_mbox_in_addr ... address of the mailbox
     p_mbox_out_addr ... address of the mailbox
}}
  Stop
  MBI_START_ADDR := p_mbox_in_addr
  MBI_LEN_ADDR := p_mbox_in_addr + mb#MBOX_LEN_B
  MBO_START_ADDR := p_mbox_out_addr
  MBO_LEN_ADDR := p_mbox_out_addr + mb#MBOX_LEN_B
  init_const(p_key_size, p_key_addr)

  success := (cog := cognew(@aes_enc_ecb, 0) + 1)

PUB Stop
''Stop AES encoder, if any.
  if cog
    cogstop(cog~ - 1)

PRI init_const(p_key_size, p_key_addr) | i
  PSBOX_ADDR := @sbox
  PRCON_ADDR := @rcon
'  PBLOCK_ADDR := @par_block
  PEXPKEY_ADDR := @par_exp_key
  if p_key_size == 128
    PKEYSIZE := 4
    PEXPKEYSIZE := 176
    PNUMROUNDS := 10
  elseif p_key_size == 192
    PKEYSIZE := 6
    PEXPKEYSIZE := 208
    PNUMROUNDS := 12
  elseif p_key_size == 256
    PKEYSIZE := 8
    PEXPKEYSIZE := 240
    PNUMROUNDS := 14
  else
    ' internal error
  repeat i from 0 to PKEYSIZE - 1
    par_exp_key[i] := long[p_key_addr][i]

DAT
                        org     0

'
' main init --> extern, before loading the code into the cog
'
aes_enc_ecb
                        call    #exp_key
:loop
                        call    #load_state
                        mov     roundkey_addr, PEXPKEY_ADDR
                        call    #aes
                        call    #store_state
                        jmp     #:loop
aes_enc_ecb_ret         ret  ' will never return
'
' --------------------------------------------------
'
'
'
' transfer block to state, pivot it
'
load_state              ' WAIT
:wait                   rdbyte  r2, MBI_LEN_ADDR wz
        if_z            jmp     #:wait
                        ' Init
                        mov     r1, MBI_START_ADDR
                        movd    :m_rdb, #state
                        mov     counter, #STATE_LEN
                        jmp     #:loop_entry
:loop                   ' advance for next loop
                        test    counter, #3 WZ
        if_nz           add     :m_rdb, DEST_4
        if_z            sub     :m_rdb, DEST_11
                        add     r1, #1
:loop_entry             ' loop main logic
:m_rdb                  rdbyte  0-0, r1         ' dst advances 3 times and then falls back to it's initial value + 1
                        djnz    counter, #:loop
                        wrbyte  counter, MBI_LEN_ADDR ' RELEASE = write back 0
                        test    r2, BIT7 wz  ' bit 7 flags the end
        if_nz           or      end_flag, BIT7
load_state_ret          ret
'
' --------------------------------------------------
'
store_state
' WAIT
:wait                   rdbyte  r1, MBO_LEN_ADDR wz
        if_nz           jmp     #:wait
' process
                        mov     r1, MBO_START_ADDR
                        movd    :m_wrb, #state
                        mov     counter, #STATE_LEN
                        jmp     #:loop_entry
:loop
                        test    counter, #3 WZ
        if_nz           add     :m_wrb, DEST_4
        if_z            sub     :m_wrb, DEST_11
                        add     r1, #1
:loop_entry
:m_wrb                  wrbyte  0-0, r1         ' dst advances 3 times and then falls back to it's initial value + 1
                        djnz    counter, #:loop
                        ' RELEASE = write back len or len || bit 7
                        wrbyte  end_flag, MBO_LEN_ADDR
store_state_ret         ret
'
' --------------------------------------------------
'
MBI_START_ADDR          long  0
MBI_LEN_ADDR            long  0
MBO_START_ADDR          long  0
MBO_LEN_ADDR            long  0
end_flag                long  mb#MBOX_LEN_B
'
' --------------------------------------------------
'
aes
                        call    #add_round_key
                        mov     rd_ctr, PNUMROUNDS
                        sub     rd_ctr, #1
:round
                        call    #sub_bytes
                        call    #shift_rows
                        call    #mix_cols
                        call    #add_round_key
                        djnz    rd_ctr, #:round
                        ' final_round
                        call    #sub_bytes
                        call    #shift_rows
                        call    #add_round_key
aes_ret                 ret
rd_ctr        long  0
'
' --------------------------------------------------
'
add_round_key
                        movd    :x, #state
                        mov     counter, #STATE_LEN
                        jmp     #:loop_entry
:loop
                        test    counter, #3 WZ
        if_nz           add     roundkey_addr, #4
        if_z            sub     roundkey_addr, #11
:loop_entry
                        rdbyte  r3, roundkey_addr
:x                      xor     0-0, r3
                        add     :x, DEST_1
                        djnz    counter, #:loop
                        add     roundkey_addr, #1
add_round_key_ret       ret
'
' --------------------------------------------------
'
sub_bytes
                        movs    :a, #state
                        movd    :rdb, #state
                        mov     counter, #STATE_LEN
:loop                   mov     r1, PSBOX_ADDR
:a                      add     r1, 0-0
:rdb                    rdbyte  0-0, r1
                        add     :a, #1
                        add     :rdb, DEST_1
                        djnz    counter, #:loop
sub_bytes_ret           ret
'
' --------------------------------------------------
'
shift_rows
                        mov     p1, #state
                        add     p1, #12
                        mov     p2, #3
:loop                   call    #shift_row
                        sub     p1, #4
                        djnz    p2, #:loop
shift_rows_ret          ret
'
' ---
'
shift_row  ' parameter: p1(in) = line in state; p2(in) = no. of shifts
                        mov     r2, p2  ' r2 = local counter
                        movs    :to_tmp, p1
                        movd    :from_tmp, p1
                        add     :from_tmp, DEST_3
:shifts                 movd    :shift1, p1
                        movs    :shift1, p1
                        add     :shift1, #1
                        mov     counter, #3
:to_tmp                 mov     r1, 0-0
:shift1                 mov     0-0, 0-0
                        add     :shift1, DEST_1_SRC_1
                        djnz    counter, #:shift1
:from_tmp               mov     0-0, r1
                        djnz    r2, #:shifts
shift_row_ret           ret
'
' --------------------------------------------------
'
mix_cols
                        mov     m_ctr1, #4
                        movs    :to_col, #state
                        movd    :to_col, #m_col
                        movs    :from_prd, #m_prd
                        movd    :from_prd, #state
:loop_cols
                        mov     m_ctr2, #4
:loop_to_col
:to_col                 mov     0-0, 0-0
                        add     :to_col, DEST_1_SRC_4
                        djnz    m_ctr2, #:loop_to_col
                        sub     :to_col, DEST_4_SRC_15

                        call    #mix_col_enc

                        mov     m_ctr2, #4
:loop_from_prd
:from_prd               mov     0-0, 0-0
                        add     :from_prd, DEST_4_SRC_1
                        djnz    m_ctr2, #:loop_from_prd
                        sub     :from_prd, DEST_15_SRC_4
                        djnz    m_ctr1, #:loop_cols
mix_cols_ret            ret
'
' ------
'
mix_col_enc  ' parameter: m_col(in) = col to be mixed for encoding, m_prd(out) = product
                        mov     m_ctr3, #4
                        movd    :res, #m_prd
                        movd    :init_res, #m_prd
:loop_prd
                        movs    :fill_p1, #m_col
:init_res               mov     0-0, #0
                        mov     m_ctr4, #4
:loop_val
:fill_p1                mov     p1, 0-0
                        mov     p2, enc_factors
                        and     p2, #$FF
                        call    #mul_gal_123
:res                    xor     0-0, p1  ' writes to m_prd
                        add     :fill_p1, #1
                        ror     enc_factors, #8
                        djnz    m_ctr4, #:loop_val

                        add     :res, DEST_1
                        add     :init_res, DEST_1
                        rol     enc_factors, #8
                        djnz    m_ctr3, #:loop_prd
mix_col_enc_ret         ret
enc_factors   long $01010302
'
' ------
'
mul_gal_123  ' parameters: p1(in/out) = addr. to number (0..255), p2(in) = number (1..3)
                        cmp     p2, #1 wz
              if_z      jmp     #mul_gal_123_ret
                        mov     r1, p1
                        ' XTIME(x) ((x << 1) ^ ((x & 0x80) ? 0x1b : 0x00))
                        shl     p1, #1
                        test    p1, #$100 wz
              if_nz     xor     p1, #$11b
                        cmp     p2, #2 wz
              if_nz     xor     p1, r1
mul_gal_123_ret         ret
'
' --------------------------------------------------
'
exp_key
' every column of the expanded key will be stored in one long
'   start with input key stored in expanded key
                        mov     cur_size, PEXPKEYSIZE
                        shr     cur_size, #2  ' PEXPKEYSIZE is in bytes, loop counter in longs!!
                        sub     cur_size, PKEYSIZE
                        mov     ek_pos_mks, PEXPKEY_ADDR
                        mov     ek_pos, PKEYSIZE
                        shl     ek_pos, #2  ' PKEYSIZE is in longs!!
                        add     ek_pos, PEXPKEY_ADDR
                        sub     ek_pos, #4
                        rdlong  p1, ek_pos  ' p1 ist the working register
                        mov     r2, PKEYSIZE  ' r2 is for a simple modulo operation
                        mov     rcon_addr, PRCON_ADDR
:loop_word                                  ' in the loop p1 contains the previous value [cur_word - 1]
                        add     ek_pos, #4
'                     test    cur_size mod keysize wz
                        cmpsub  r2, PKEYSIZE wz
              if_z      call    #key_sched_core ' Param: p1
                        call    #extra256
'                     xor     p1 ^ [cur_word - keysize] --> [cur_word]
                        rdlong  r1, ek_pos_mks
                        xor     p1, r1
                        add     ek_pos_mks, #4
                        wrlong  p1, ek_pos
'
                        add     r2, #1
                        djnz    cur_size, #:loop_word
exp_key_ret             ret
'
ek_pos        long  0
ek_pos_mks    long  0
cur_size      long  0
'
' - - - - -
'
              ' if 256b call sbox(p1)
extra256
              if_always jmp     #extra256_ret
                        test    r2, #3 wz
              if_z      call    #sub_bytes_long
extra256_ret            ret
'
' - - - - -
'
key_sched_core  ' parameters: p1(in/out) = 1 col of exp. key
                        ror     p1, #8
                        call    #sub_bytes_long  ' param. p1
                        ' xor mit rcon
                        rdbyte  r3, rcon_addr
                        add     rcon_addr, #1
                        xor     p1, r3
key_sched_core_ret      ret
rcon_addr     long  0
'
' - - - - -
'
sub_bytes_long  ' parameters: p1(in/out) = 1 long (4 bytes) to be substituted with sbox
                        mov     m_ctr1, #4
:loop                   mov     r3, p1
                        and     r3, M_LOWBYTE
                        add     r3, PSBOX_ADDR
                        rdbyte  r3, r3
                        andn    p1, M_LOWBYTE
                        or      p1, r3
                        ror     p1, #8
                        djnz    m_ctr1, #:loop
sub_bytes_long_ret      ret
'
' --------------------------------------------------
'
' constants
DEST_1        long  1<<9
DEST_3        long  3<<9
DEST_4        long  4<<9
DEST_11       long  11<<9
DEST_1_SRC_1  long  (1<<9)+1
DEST_1_SRC_4  long  (1<<9)+4
DEST_4_SRC_15 long  (4<<9)+15
DEST_4_SRC_1  long  (4<<9)+1
DEST_15_SRC_4 long  (15<<9)+4
M_LOWBYTE     long  $FF
BIT7          long  1 << 7
' temp registers
r1            long  0           ' temp. variable. Be careful when calling subroutines
r2            long  0
r3            long  0
p1            long  0           ' arbitrary parameter for calls
p2            long  0
counter       long  0
round_ctr     long  0  ' counter for the rounds, will count down for djnz!
roundkey_addr long  0  ' hub ram address of the round key for the current round
' parameter block, don't change the variables and their order
parameters
PSBOX_ADDR    long  0  ' hub RAM address of sbox
PRCON_ADDR    long  0  ' hub RAM address of rcon[1..14]
PEXPKEY_ADDR  long  0  ' hub RAM address of expanded key
PKEYSIZE      long  0  ' chiffre key size in longs
PEXPKEYSIZE   long  0  ' expanded key size in bytes
PNUMROUNDS    long  0
' parameter block ends here
'''PBLOCK_ADDR   long  0
state         long  0[STATE_LEN]

m_col         long  0[4]        ' column for galois multiplication
m_prd         long  0[4]        ' product of galois multiplication
m_ctr1        long  0
m_ctr2        long  0
m_ctr3        long  0
m_ctr4        long  0

        fit   496


DAT
'
' ________________________________________________
'
sbox    byte $63, $7c, $77, $7b, $f2, $6b, $6f, $c5, $30, $01, $67, $2b, $fe, $d7, $ab, $76 ' 0
sb_1    byte $ca, $82, $c9, $7d, $fa, $59, $47, $f0, $ad, $d4, $a2, $af, $9c, $a4, $72, $c0 ' 1
sb_2    byte $b7, $fd, $93, $26, $36, $3f, $f7, $cc, $34, $a5, $e5, $f1, $71, $d8, $31, $15 ' 2
sb_3    byte $04, $c7, $23, $c3, $18, $96, $05, $9a, $07, $12, $80, $e2, $eb, $27, $b2, $75 ' 3
sb_4    byte $09, $83, $2c, $1a, $1b, $6e, $5a, $a0, $52, $3b, $d6, $b3, $29, $e3, $2f, $84 ' 4
sb_5    byte $53, $d1, $00, $ed, $20, $fc, $b1, $5b, $6a, $cb, $be, $39, $4a, $4c, $58, $cf ' 5
sb_6    byte $d0, $ef, $aa, $fb, $43, $4d, $33, $85, $45, $f9, $02, $7f, $50, $3c, $9f, $a8 ' 6
sb_7    byte $51, $a3, $40, $8f, $92, $9d, $38, $f5, $bc, $b6, $da, $21, $10, $ff, $f3, $d2 ' 7
sb_8    byte $cd, $0c, $13, $ec, $5f, $97, $44, $17, $c4, $a7, $7e, $3d, $64, $5d, $19, $73 ' 8
sb_9    byte $60, $81, $4f, $dc, $22, $2a, $90, $88, $46, $ee, $b8, $14, $de, $5e, $0b, $db ' 9
sb_A    byte $e0, $32, $3a, $0a, $49, $06, $24, $5c, $c2, $d3, $ac, $62, $91, $95, $e4, $79 ' A
sb_B    byte $e7, $c8, $37, $6d, $8d, $d5, $4e, $a9, $6c, $56, $f4, $ea, $65, $7a, $ae, $08 ' B
sb_C    byte $ba, $78, $25, $2e, $1c, $a6, $b4, $c6, $e8, $dd, $74, $1f, $4b, $bd, $8b, $8a ' C
sb_D    byte $70, $3e, $b5, $66, $48, $03, $f6, $0e, $61, $35, $57, $b9, $86, $c1, $1d, $9e ' D
sb_E    byte $e1, $f8, $98, $11, $69, $d9, $8e, $94, $9b, $1e, $87, $e9, $ce, $55, $28, $df ' E
sb_F    byte $8c, $a1, $89, $0d, $bf, $e6, $42, $68, $41, $99, $2d, $0f, $b0, $54, $bb, $16 ' F

rcon    byte $01, $02, $04, $08, $10, $20, $40, $80, $1b, $36



