
'controller
CON

_CLKMODE     = XTAL1 + PLL16X
_XINFREQ     = 5_000_000

OBJ
  mb: "mbox16"
  hs: "hubstrm"
  ae: "a128"
  ios: "reg-ios"
  pt: "pterm"

CON
  DATA_LEN = 16

  #0, NOTHING, _IOS, _PTERM
  USE_IO = NOTHING  ' set to 0 for debugging in GEAR

VAR
  long mbox1[mb#MBOX_LEN_L]
  long mbox1_len  ' must follow to mbox!
  long mbox2[mb#MBOX_LEN_L]
  long mbox2_len  ' must follow to mbox!
  long mbox_endflag

PUB main
  case USE_IO
    _IOS:
      ios.startram
      ios.printnl
      ios.print(string("Press any key to start"))
      ios.keywait
    _PTERM:
      pt.start(9600)
      pt.NewLine
      pt.str(string("Press any key to start"))
      pt.CharIn
  testcase(0, @tc0)
  case USE_IO
    _IOS:
      ios.printnl
      ios.print(string("Press any key to quit"))
      ios.keywait
      ios.printnl
      ios.stop
    _PTERM:
      pt.NewLine
      pt.str(string("Press any key to quit"))
      pt.CharIn
      pt.NewLine
      pt.stop

CON
  #0, EXP_OUT_L,
      EXP_OUT,
      AES_MODE,
      BLOCK_MODE,
      INIT_VEC,
      PTXT_L,
      PTXT,
      KEY

PRI testcase(num_tc, tc)

  ' select_testcase --> extern
  print_tc(num_tc, tc)
  ' fill_input
  ' start_test
  mb.mbox16_init(@mbox1)
  mb.mbox16_init(@mbox2)
  mbox_endflag := 0
  hs.Start(@mbox1, @mbox2, @mbox_endflag, long[tc][PTXT], @data_out, long[tc][PTXT_L])
  ae.Start(@mbox1, @mbox2, long[tc][AES_MODE], long[tc][KEY])
  ' await_result
  repeat while mbox_endflag == 0
'  waitcnt(10_000_000 + cnt)
  hs.stop
  ae.stop

  ' check_result
  check_result(long[tc][EXP_OUT_L], long[tc][EXP_OUT], @data_out)

PRI print_tc(num_tc, tc)
  case USE_IO
    _IOS:
      ios.printnl
      ios.print(string("TC#"))
      ios.printdec(num_tc)
      ios.print(string(": AES"))
      ios.printdec(long[tc][AES_MODE])
      ios.print(string(", block mode "))
      ios.print(lookup(long[tc][BLOCK_MODE]: string("ECB"), string("unknown")))
    _PTERM:
      pt.NewLine
      pt.linefeed
      pt.str(string("TC#"))
      pt.dec(num_tc)
      pt.str(string(": AES"))
      pt.dec(long[tc][AES_MODE])
      pt.str(string(", block mode "))
      pt.str(lookup(long[tc][BLOCK_MODE]: string("ECB"), string("unknown")))

PRI check_result(len, expected, computed) | failed, i
  failed := 0

  repeat i from 0 to len - 1
    if byte[expected][i] <> byte[computed][i]
      failed := 1
      quit
  if failed == 1
    case USE_IO
      _IOS:
        ios.setcolor(2)
        ios.print(string(" ...failed"))
        ios.printnl
        ios.print(string("  exp: "))
        repeat i from 0 to len - 1
          ios.printhex(byte[expected][i], 2)
          ios.printchar(" ")
        ios.printnl
        ios.print(string("  got: "))
        repeat i from 0 to len - 1
          ios.printhex(byte[computed][i], 2)
          ios.printchar(" ")
      _PTERM:
        pt.str(string(" ...failed"))
        pt.NewLine
        pt.str(string("  exp: "))
        repeat i from 0 to len - 1
          pt.hex(byte[expected][i], 2)
          pt.char(" ")
        pt.NewLine
        pt.str(string("  got: "))
        repeat i from 0 to len - 1
          pt.hex(byte[computed][i], 2)
          pt.char(" ")
  else
    case USE_IO
      _IOS:
        ios.setcolor(1)
        ios.print(string(" ...passed"))
      _PTERM:
        pt.str(string(" ...passed"))
  case USE_IO
    _IOS:
      ios.setcolor(0)
      ios.printnl
    _PTERM:
      pt.NewLine

CON
  #1,   ECB

DAT
' Test case data structure
'       expected_output_len     long
'       expected_output         byte[expected_output_len]
'       aes_mode                long ' possible values: 128, 192, 256
'       mode_of_operation       long ' 1 ... ECB
'       init_vector             byte[]
'       plaintext_length        long
'       plaintext               byte[plaintext_length]
'       key                     byte[]

tc0           long      16, @@@tc0_exp_out, 128, ECB, 0, 16, @@@tc0_ptxt, @@@tc0_key

tc0_exp_out   byte      $39, $25, $84, $1d, $02, $dc, $09, $fb, $dc, $11, $85, $97, $19, $6a, $0b, $32
tc0_ptxt      byte      $32, $43, $f6, $a8, $88, $5a, $30, $8d, $31, $31, $98, $a2, $e0, $37, $07, $34
tc0_key       byte      $2b, $7e, $15, $16, $28, $ae, $d2, $a6, $ab, $f7, $15, $88, $09, $cf, $4f, $3c

DAT

dummy         byte      $FF, $FF
data_out      byte      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
check_empty   byte      0, 0, 0, 0, 0, 0, 0, 0, 0, 0
  ' must be and stay empty (=0)


